scripts: mkbootimg_rpi4: Fix url path to tizen_7.0
[platform/kernel/linux-rpi.git] / fs / dax.c
index 994ab66..d5d7b93 100644 (file)
--- a/fs/dax.c
+++ b/fs/dax.c
@@ -144,6 +144,16 @@ struct wait_exceptional_entry_queue {
        struct exceptional_entry_key key;
 };
 
+/**
+ * enum dax_wake_mode: waitqueue wakeup behaviour
+ * @WAKE_ALL: wake all waiters in the waitqueue
+ * @WAKE_NEXT: wake only the first waiter in the waitqueue
+ */
+enum dax_wake_mode {
+       WAKE_ALL,
+       WAKE_NEXT,
+};
+
 static wait_queue_head_t *dax_entry_waitqueue(struct xa_state *xas,
                void *entry, struct exceptional_entry_key *key)
 {
@@ -182,7 +192,8 @@ static int wake_exceptional_entry_func(wait_queue_entry_t *wait,
  * The important information it's conveying is whether the entry at
  * this index used to be a PMD entry.
  */
-static void dax_wake_entry(struct xa_state *xas, void *entry, bool wake_all)
+static void dax_wake_entry(struct xa_state *xas, void *entry,
+                          enum dax_wake_mode mode)
 {
        struct exceptional_entry_key key;
        wait_queue_head_t *wq;
@@ -196,7 +207,7 @@ static void dax_wake_entry(struct xa_state *xas, void *entry, bool wake_all)
         * must be in the waitqueue and the following check will see them.
         */
        if (waitqueue_active(wq))
-               __wake_up(wq, TASK_NORMAL, wake_all ? 0 : 1, &key);
+               __wake_up(wq, TASK_NORMAL, mode == WAKE_ALL ? 0 : 1, &key);
 }
 
 /*
@@ -264,11 +275,11 @@ static void wait_entry_unlocked(struct xa_state *xas, void *entry)
        finish_wait(wq, &ewait.wait);
 }
 
-static void put_unlocked_entry(struct xa_state *xas, void *entry)
+static void put_unlocked_entry(struct xa_state *xas, void *entry,
+                              enum dax_wake_mode mode)
 {
-       /* If we were the only waiter woken, wake the next one */
        if (entry && !dax_is_conflict(entry))
-               dax_wake_entry(xas, entry, false);
+               dax_wake_entry(xas, entry, mode);
 }
 
 /*
@@ -286,7 +297,7 @@ static void dax_unlock_entry(struct xa_state *xas, void *entry)
        old = xas_store(xas, entry);
        xas_unlock_irq(xas);
        BUG_ON(!dax_is_locked(old));
-       dax_wake_entry(xas, entry, false);
+       dax_wake_entry(xas, entry, WAKE_NEXT);
 }
 
 /*
@@ -477,10 +488,11 @@ static void *grab_mapping_entry(struct xa_state *xas,
                struct address_space *mapping, unsigned int order)
 {
        unsigned long index = xas->xa_index;
-       bool pmd_downgrade = false; /* splitting PMD entry into PTE entries? */
+       bool pmd_downgrade;     /* splitting PMD entry into PTE entries? */
        void *entry;
 
 retry:
+       pmd_downgrade = false;
        xas_lock_irq(xas);
        entry = get_unlocked_entry(xas, order);
 
@@ -524,7 +536,7 @@ retry:
 
                dax_disassociate_entry(entry, mapping, false);
                xas_store(xas, NULL);   /* undo the PMD join */
-               dax_wake_entry(xas, entry, true);
+               dax_wake_entry(xas, entry, WAKE_ALL);
                mapping->nrexceptional--;
                entry = NULL;
                xas_set(xas, index);
@@ -559,8 +571,11 @@ fallback:
 }
 
 /**
* dax_layout_busy_page - find first pinned page in @mapping
  * @mapping: address space to scan for a page with ref count > 1
+ * @start: Starting offset. Page containing 'start' is included.
+ * @end: End offset. Page containing 'end' is included. If 'end' is LLONG_MAX,
+ *       pages from 'start' till the end of file are included.
  *
  * DAX requires ZONE_DEVICE mapped pages. These pages are never
  * 'onlined' to the page allocator so they are considered idle when
@@ -573,12 +588,15 @@ fallback:
  * to be able to run unmap_mapping_range() and subsequently not race
  * mapping_mapped() becoming true.
  */
-struct page *dax_layout_busy_page(struct address_space *mapping)
+struct page *dax_layout_busy_page_range(struct address_space *mapping,
+                                       loff_t start, loff_t end)
 {
-       XA_STATE(xas, &mapping->i_pages, 0);
        void *entry;
        unsigned int scanned = 0;
        struct page *page = NULL;
+       pgoff_t start_idx = start >> PAGE_SHIFT;
+       pgoff_t end_idx;
+       XA_STATE(xas, &mapping->i_pages, start_idx);
 
        /*
         * In the 'limited' case get_user_pages() for dax is disabled.
@@ -589,6 +607,11 @@ struct page *dax_layout_busy_page(struct address_space *mapping)
        if (!dax_mapping(mapping) || !mapping_mapped(mapping))
                return NULL;
 
+       /* If end == LLONG_MAX, all pages from start to till end of file */
+       if (end == LLONG_MAX)
+               end_idx = ULONG_MAX;
+       else
+               end_idx = end >> PAGE_SHIFT;
        /*
         * If we race get_user_pages_fast() here either we'll see the
         * elevated page count in the iteration and wait, or
@@ -596,22 +619,22 @@ struct page *dax_layout_busy_page(struct address_space *mapping)
         * against is no longer mapped in the page tables and bail to the
         * get_user_pages() slow path.  The slow path is protected by
         * pte_lock() and pmd_lock(). New references are not taken without
* holding those locks, and unmap_mapping_range() will not zero the
         * pte or pmd without holding the respective lock, so we are
         * guaranteed to either see new references or prevent new
         * references from being established.
         */
-       unmap_mapping_range(mapping, 0, 0, 0);
+       unmap_mapping_pages(mapping, start_idx, end_idx - start_idx + 1, 0);
 
        xas_lock_irq(&xas);
-       xas_for_each(&xas, entry, ULONG_MAX) {
+       xas_for_each(&xas, entry, end_idx) {
                if (WARN_ON_ONCE(!xa_is_value(entry)))
                        continue;
                if (unlikely(dax_is_locked(entry)))
                        entry = get_unlocked_entry(&xas, 0);
                if (entry)
                        page = dax_busy_page(entry);
-               put_unlocked_entry(&xas, entry);
+               put_unlocked_entry(&xas, entry, WAKE_NEXT);
                if (page)
                        break;
                if (++scanned % XA_CHECK_SCHED)
@@ -625,6 +648,12 @@ struct page *dax_layout_busy_page(struct address_space *mapping)
        xas_unlock_irq(&xas);
        return page;
 }
+EXPORT_SYMBOL_GPL(dax_layout_busy_page_range);
+
+struct page *dax_layout_busy_page(struct address_space *mapping)
+{
+       return dax_layout_busy_page_range(mapping, 0, LLONG_MAX);
+}
 EXPORT_SYMBOL_GPL(dax_layout_busy_page);
 
 static int __dax_invalidate_entry(struct address_space *mapping,
@@ -647,7 +676,7 @@ static int __dax_invalidate_entry(struct address_space *mapping,
        mapping->nrexceptional--;
        ret = 1;
 out:
-       put_unlocked_entry(&xas, entry);
+       put_unlocked_entry(&xas, entry, WAKE_ALL);
        xas_unlock_irq(&xas);
        return ret;
 }
@@ -793,12 +822,12 @@ static void dax_entry_mkclean(struct address_space *mapping, pgoff_t index,
                address = pgoff_address(index, vma);
 
                /*
* Note because we provide range to follow_pte_pmd it will
* call mmu_notifier_invalidate_range_start() on our behalf
* before taking any lock.
                 */
-               if (follow_pte_pmd(vma->vm_mm, address, &range,
-                                  &ptep, &pmdp, &ptl))
+               if (follow_invalidate_pte(vma->vm_mm, address, &range, &ptep,
+                                         &pmdp, &ptl))
                        continue;
 
                /*
@@ -920,13 +949,13 @@ static int dax_writeback_one(struct xa_state *xas, struct dax_device *dax_dev,
        xas_lock_irq(xas);
        xas_store(xas, entry);
        xas_clear_mark(xas, PAGECACHE_TAG_DIRTY);
-       dax_wake_entry(xas, entry, false);
+       dax_wake_entry(xas, entry, WAKE_NEXT);
 
        trace_dax_writeback_one(mapping->host, index, count);
        return ret;
 
  put_unlocked:
-       put_unlocked_entry(xas, entry);
+       put_unlocked_entry(xas, entry, WAKE_NEXT);
        return ret;
 }
 
@@ -1037,18 +1066,18 @@ static vm_fault_t dax_load_hole(struct xa_state *xas,
        return ret;
 }
 
-int dax_iomap_zero(loff_t pos, unsigned offset, unsigned size,
-                  struct iomap *iomap)
+s64 dax_iomap_zero(loff_t pos, u64 length, struct iomap *iomap)
 {
        sector_t sector = iomap_sector(iomap, pos & PAGE_MASK);
        pgoff_t pgoff;
        long rc, id;
        void *kaddr;
        bool page_aligned = false;
-
+       unsigned offset = offset_in_page(pos);
+       unsigned size = min_t(u64, PAGE_SIZE - offset, length);
 
        if (IS_ALIGNED(sector << SECTOR_SHIFT, PAGE_SIZE) &&
-           IS_ALIGNED(size, PAGE_SIZE))
+           (size == PAGE_SIZE))
                page_aligned = true;
 
        rc = bdev_dax_pgoff(iomap->bdev, sector, PAGE_SIZE, &pgoff);
@@ -1058,8 +1087,7 @@ int dax_iomap_zero(loff_t pos, unsigned offset, unsigned size,
        id = dax_read_lock();
 
        if (page_aligned)
-               rc = dax_zero_page_range(iomap->dax_dev, pgoff,
-                                        size >> PAGE_SHIFT);
+               rc = dax_zero_page_range(iomap->dax_dev, pgoff, 1);
        else
                rc = dax_direct_access(iomap->dax_dev, pgoff, 1, &kaddr, NULL);
        if (rc < 0) {
@@ -1072,7 +1100,7 @@ int dax_iomap_zero(loff_t pos, unsigned offset, unsigned size,
                dax_flush(iomap->dax_dev, kaddr + offset, size);
        }
        dax_read_unlock(id);
-       return 0;
+       return size;
 }
 
 static loff_t
@@ -1668,7 +1696,7 @@ dax_insert_pfn_mkwrite(struct vm_fault *vmf, pfn_t pfn, unsigned int order)
        /* Did we race with someone splitting entry or so? */
        if (!entry || dax_is_conflict(entry) ||
            (order == 0 && !dax_is_pte_entry(entry))) {
-               put_unlocked_entry(&xas, entry);
+               put_unlocked_entry(&xas, entry, WAKE_NEXT);
                xas_unlock_irq(&xas);
                trace_dax_insert_pfn_mkwrite_no_entry(mapping->host, vmf,
                                                      VM_FAULT_NOPAGE);
@@ -559,8 +571,11 @@ fallback:
 
 /**
- * dax_layout_busy_page - find first pinned page in @mapping
+ * dax_layout_busy_page_range - find first pinned page in @mapping
  * @mapping: address space to scan for a page with ref count > 1
@@ -596,22 +619,22 @@ struct page *dax_layout_busy_page(struct address_space *mapping)
         * get_user_pages() slow path.  The slow path is protected by
         * pte_lock() and pmd_lock(). New references are not taken without
-        * holding those locks, and unmap_mapping_range() will not zero the
+        * holding those locks, and unmap_mapping_pages() will not zero the
         * pte or pmd without holding the respective lock, so we are
         * guaranteed to either see new references or prevent new
@@ -793,12 +822,12 @@ static void dax_entry_mkclean(struct address_space *mapping, pgoff_t index,
 
                /*
-                * Note because we provide range to follow_pte_pmd it will
-                * call mmu_notifier_invalidate_range_start() on our behalf
-                * before taking any lock.
+                * follow_invalidate_pte() will use the range to call
+                * mmu_notifier_invalidate_range_start() on our behalf before
+                * taking any lock.
                 */